@@ -7,8 +7,9 @@ http://www.graphlab.com/products/create/tutorials.html.
 
 """
 
-__all__ = ['sframe', 'sarray', 'graph']
+__all__ = ['sframe', 'sarray', 'graph', 'sketch']
 
 import sframe
 import sarray
 import graph
+import sketch
