@@ -1,5 +1,6 @@
 """
-.. warning:: This product is currently in a beta release. The API reference is subject to change.
+.. warning:: This product is currently in a beta release. The API reference is
+subject to change.
 
 This package defines the GraphLab Graph, Vertex, and Edge objects. The Graph is
 a directed graph, consisting of a set of Vertex objects and Edges that connect
@@ -10,9 +11,11 @@ import of the graphlab package.
 
 import graphlab.connect.main as glconnect
 from graphlab.data_structures.sframe import SFrame
-from graphlab.cython.graph import UnityGraphProxy
-from graphlab.util import make_internal_url, shallow_throw
+from graphlab.cython.cy_graph import UnityGraphProxy
+from graphlab.cython.context import debug_trace as cython_context
+from graphlab.util import make_internal_url
 import pandas as pd
+import numpy as np
 
 ## \internal Default column name for vertex id.
 _VID_COLUMN = '__id'
@@ -258,7 +261,6 @@ class Graph(object):
         ret = self.__proxy__.summary()
         return dict(ret.items())
 
-    @shallow_throw
     def get_vertices(self, ids=[], fields={}, format='sframe'):
         """
         Return a collection of vertices and their attributes.
@@ -317,7 +319,9 @@ class Graph(object):
         if not hasattr(ids, '__iter__'):
             ids = [ids]
 
-        sf = SFrame(_proxy=self.__proxy__.get_vertices(ids, fields))
+        with cython_context():
+            sf = SFrame(_proxy=self.__proxy__.get_vertices(ids, fields))
+
         if (format == 'sframe'):
             return sf
         elif (format == 'dataframe'):
@@ -331,7 +335,6 @@ class Graph(object):
         else:
             raise ValueError("Invalid format specifier")
 
-    @shallow_throw
     def get_edges(self, src_ids=[], dst_ids=[], fields={}, format='sframe'):
         """
         Return a collection of edges and their attributes. This function can be
@@ -415,7 +418,9 @@ class Graph(object):
         if len(dst_ids) == 0 and len(src_ids) > 0:
             dst_ids = [None] * len(src_ids)
 
-        sf = SFrame(_proxy=self.__proxy__.get_edges(src_ids, dst_ids, fields))
+        with cython_context():
+            sf = SFrame(_proxy=self.__proxy__.get_edges(src_ids, dst_ids, fields))
+
         if (format == 'sframe'):
             return sf
         if (format == 'dataframe'):
@@ -428,7 +433,6 @@ class Graph(object):
         else:
             raise ValueError("Invalid format specifier")
 
-    @shallow_throw
     def add_vertices(self, vertices, vid_field=None, format='auto'):
         """
         Add one or many vertices to the graph. Vertices should be input as a
@@ -475,38 +479,38 @@ class Graph(object):
             else:
                 raise TypeError('Vertices type %s is Not supported.' % str(type(vertices)))
 
-        if format == 'vertex':
-            return Graph(_proxy=self.__proxy__.add_vertices(
-                  _vertex_list_to_dataframe([vertices], '__id'), '__id'))
-        elif format == 'list':
-            return Graph(_proxy=self.__proxy__.add_vertices(
-                         _vertex_list_to_dataframe(vertices, '__id'), '__id'))
-        elif format == 'dataframe':
-            if vid_field is None:
-                # using the dataframe index as vertex id
-                if vertices.index.is_unique:
-                    if not ("index" in vertices.columns):
-                        # pandas reset_index() will insert a new column of name "index".
-                        return Graph(_proxy=self.__proxy__.add_vertices(vertices.reset_index(), "index"))
+        with cython_context():
+            if format == 'vertex':
+                return Graph(_proxy=self.__proxy__.add_vertices(
+                             _vertex_list_to_dataframe([vertices], '__id'), '__id'))
+            elif format == 'list':
+                return Graph(_proxy=self.__proxy__.add_vertices(
+                             _vertex_list_to_dataframe(vertices, '__id'), '__id'))
+            elif format == 'dataframe':
+                if vid_field is None:
+                    # using the dataframe index as vertex id
+                    if vertices.index.is_unique:
+                        if not ("index" in vertices.columns):
+                            # pandas reset_index() will insert a new column of name "index".
+                            return Graph(_proxy=self.__proxy__.add_vertices(vertices.reset_index(), "index"))
+                        else:
+                            # pandas reset_index() will insert a new column of name "level_0" if there exists a column named "index".
+                            return Graph(_proxy=self.__proxy__.add_vertices(vertices.reset_index(), "level_0"))
                     else:
-                        # pandas reset_index() will insert a new column of name "level_0" if there exists a column named "index".
-                        return Graph(_proxy=self.__proxy__.add_vertices(vertices.reset_index(), "level_0"))
+                        raise ValueError("Index of the vertices dataframe is not unique, \
+                                try specifying vid_field name to use a column for vertex ids.")
                 else:
-                    raise ValueError("Index of the vertices dataframe is not unique, \
-                            try specifying vid_field name to use a column for vertex ids.")
+                    # using the given column as vertex id
+                    return Graph(_proxy=self.__proxy__.add_vertices(vertices, vid_field))
+            elif format == 'sframe':
+                if vid_field is not None:
+                    return Graph(_proxy=self.__proxy__.add_vertices_from_sframe(vertices.__proxy__, vid_field))
+                    pass
+                else:
+                    raise ValueError("vid_field must be specified for sframe input.")
             else:
-                # using the given column as vertex id
-                return Graph(_proxy=self.__proxy__.add_vertices(vertices, vid_field))
-        elif format == 'sframe':
-            if vid_field is not None:
-                return Graph(_proxy=self.__proxy__.add_vertices_from_sframe(vertices.__proxy__, vid_field))
-                pass
-            else:
-                raise ValueError("vid_field must be specified for sframe input.")
-        else:
-            raise TypeError('Vertices type %s is Not supported.' % str(type(vertices)))
+                raise TypeError('Vertices type %s is Not supported.' % str(type(vertices)))
 
-    @shallow_throw
     def add_edges(self, edges, src_field=None, dst_field=None, format='auto'):
         """
         Add one or many edges to the graph. Edges should be input as a list of
@@ -561,32 +565,33 @@ class Graph(object):
             else:
                 raise TypeError('Vertices type %s is Not supported.' % str(type(edges)))
 
-        if format == 'edge':
-            return Graph(_proxy=self.__proxy__.add_edges(
-                _edge_list_to_dataframe([edges], '__src_vid', '__dst_vid'), '__src_vid', '__dst_vid'))
-        elif format == 'list':
-            return Graph(_proxy=self.__proxy__.add_edges(
-                _edge_list_to_dataframe(edges, '__src_vid', '__dst_vid'), '__src_vid', '__dst_vid'))
-        elif format == 'dataframe':
-            if src_field is None:
-                raise ValueError("src_field must be specified for Pandas input")
-            if dst_field is None:
-                raise ValueError("dst_field must be specified for Pandas input")
-            return Graph(_proxy=self.__proxy__.add_edges(edges, src_field, dst_field))
-        elif format == 'sframe':
-            if src_field is None:
-                raise ValueError("src_field must be specified for SFrame input")
-            if dst_field is None:
-                raise ValueError("dst_field must be specified for SFrame input")
-            return Graph(_proxy=self.__proxy__.add_edges_from_sframe(edges.__proxy__, src_field, dst_field))
-        else:
-            raise TypeError('Edges type %s is Not supported.' % str(type(edges)))
+        with cython_context():
+            if format == 'edge':
+                return Graph(_proxy=self.__proxy__.add_edges(
+                    _edge_list_to_dataframe([edges], '__src_vid', '__dst_vid'), '__src_vid', '__dst_vid'))
+            elif format == 'list':
+                return Graph(_proxy=self.__proxy__.add_edges(
+                    _edge_list_to_dataframe(edges, '__src_vid', '__dst_vid'), '__src_vid', '__dst_vid'))
+            elif format == 'dataframe':
+                if src_field is None:
+                    raise ValueError("src_field must be specified for Pandas input")
+                if dst_field is None:
+                    raise ValueError("dst_field must be specified for Pandas input")
+                return Graph(_proxy=self.__proxy__.add_edges(edges, src_field, dst_field))
+            elif format == 'sframe':
+                if src_field is None:
+                    raise ValueError("src_field must be specified for SFrame input")
+                if dst_field is None:
+                    raise ValueError("dst_field must be specified for SFrame input")
+                return Graph(_proxy=self.__proxy__.add_edges_from_sframe(edges.__proxy__, src_field, dst_field))
+            else:
+                raise TypeError('Edges type %s is Not supported.' % str(type(edges)))
 
     def get_fields(self):
         """Return a list of vertex and edge attribute fields in the graph."""
-        return self.__proxy__.get_fields()
+        with cython_context():
+            return self.__proxy__.get_fields()
 
-    @shallow_throw
     def select_fields(self, fields):
         """
         Return a new graph with only the selected fields.
@@ -607,9 +612,9 @@ class Graph(object):
             fields.
 
         """
-        return Graph(_proxy=self.__proxy__.select_fields(fields))
+        with cython_context():
+            return Graph(_proxy=self.__proxy__.select_fields(fields))
 
-    @shallow_throw
     def delete_field(self, field):
         """
         Delete a field in the graph. The field is removed from all vertices and
@@ -633,9 +638,9 @@ class Graph(object):
         >>> g2 = g.delete_field('woof')
         >>> g2.get_vertices()
         """
-        return Graph(_proxy=self.__proxy__.delete_field(field))
+        with cython_context():
+            return Graph(_proxy=self.__proxy__.delete_field(field))
 
-    @shallow_throw
     def copy_field(self, src_field, new_field):
         """
         Create a new field by copying an existing field. All vertices and edges
@@ -663,9 +668,9 @@ class Graph(object):
         >>> g2 = g.copy_field('woof', 'bark')
         >>> g2.get_vertices()
         """
-        return Graph(_proxy=self.__proxy__.copy_field(src_field, new_field))
+        with cython_context():
+            return Graph(_proxy=self.__proxy__.copy_field(src_field, new_field))
 
-    @shallow_throw
     def save(self, filename, format="binary"):
         """
         Save the graph to disk. If the graph is saved in binary format, the
@@ -709,18 +714,19 @@ class Graph(object):
         self.__save_graph(make_internal_url(filename), format)
 
     def __save_graph(self, filename, format):
-        if format == "json":
-            assert filename.endswith('.json')
-            self.__proxy__.save_graph_as_json(filename)
-        elif format == "binary":
-            assert filename.endswith('.graph')
-            self.__proxy__.save_graph(filename)
-        else:
-            raise ValueError("Unsupported format: " + format)
+        with cython_context():
+            if format == "json":
+                assert filename.endswith('.json')
+                self.__proxy__.save_graph_as_json(filename)
+            elif format == "binary":
+                assert filename.endswith('.graph')
+                self.__proxy__.save_graph(filename)
+            else:
+                raise ValueError("Unsupported format: " + format)
 
     def show(self, vlabel=None, vcolor=[0.6, 0., 0.6], highlight=[],
              highlight_color=[1., 0.5, 0.], node_size=300, elabel=None,
-             ecolor='gray', ewidth=3, v_offset=0.03, h_offset=0.):
+             ecolor='gray', ewidth=3, v_offset=0.03, h_offset=0., arrows=False):
         """
         Draw the graph using the `NetworkX <http://networkx.github.io/>`_ spring
         layout. Beacause drawing big graphs is messy and resource consuming,
@@ -728,40 +734,45 @@ class Graph(object):
 
         Parameters
         ----------
-        vlabel : string
+        vlabel : string, optional
             Field name for the label on each vertex. The default is None, which
             omits vertex labels. Set to 'id' to use the vertex ID as the label.
 
-        vcolor : list
+        vcolor : list, optional
             RGB triple for the primary vertex color. Default is orange.
 
-        highlight : list
+        highlight : list, optional
             Vertex IDs to highlight in a different color.
 
-        highlight_color : list
+        highlight_color : list, optional
             RGB triple for the color of highlighted vertices. Default is purple.
 
-        node_size : int
+        node_size : int, optional
             Size of plotted vertices.
 
-        elabel : string
+        elabel : string, optional
             Field name for the label on each edge.
 
-        ecolor : string
+        ecolor : string, optional
             Edge color.
 
-        ewidth : int
+        ewidth : int, optional
             Edge width.
 
-        v_offset : float
+        v_offset : float, optional
             Vertical offset of vertex labels, as a fraction of total plot
             height. For example, the default of 0.03 moves the label 3% of the
             plot height higher in the canvas.
 
-        h_offset : float
+        h_offset : float, optional
             Horizontal offset of vertex labels, as a fraction of total plot
             width. For example, an offset of 0.03 moves the label 3% of the plot
             width to the right. Default is 0.0.
+
+        arrows : bool, optional
+            If True, draw 'arrows' indicating edge direction. In place of real
+            arrows, NetworkX indicates edge direction by drawing a wider line
+            segment adjacent to the source node.
         """
 
         ## Error trapping
@@ -774,7 +785,7 @@ class Graph(object):
             raise RuntimeError("Cannot show graph with too many edges")
 
         ## Make the NetworkX graph
-        nxgraph = nx.Graph()
+        nxgraph = nx.DiGraph()
         vertices = self.get_vertices(format='list')
         edges = self.get_edges(format='list')
         node_color = []
@@ -800,7 +811,7 @@ class Graph(object):
                                alpha=0.7)
 
         nx.draw_networkx_edges(nxgraph, pos, edge_color=ecolor, width=ewidth,
-                               alpha=0.5)
+                               alpha=0.5, arrows=arrows)
 
         ## Draw vertex labels
         if vlabel:
@@ -830,8 +841,83 @@ class Graph(object):
         plt.axis('off')
         plt.show()
 
+    def get_neighborhood(self, ids, radius=1, full_subgraph=True):
+        """
+        Return a graph containing the neighborhood around a set of target vertices,
+        ignoring edge directions. Note that setting radius greater than two
+        often results in a time-consuming query for a very large subgraph.
+
+        Parameters
+        ----------
+        ids : list [int | float | str]
+            List of target vertex IDs.
+
+        radius : int, optional
+            Radius of the neighborhood to return. Setting radius larger
+            than 2 may result in a very large subgraph.
+
+        full_subgraph : bool, optional If True, return all edges between
+            vertices in a target node's neighborhood, also known as the subgraph
+            induced by the target node's neighbors, or the egocentric network
+            for the target node. If False, return only edges on paths of length
+            <= radius from the target node, also known as the reachability
+            graph.
+
+        Returns
+        -------
+        out : Graph
+            The subgraph with the neighborhoods around the target vertices.
+
+        Examples
+        --------
+
+        References
+        ----------
+        - Marsden, P. (2002) `Egocentric and sociocentric measures of network
+          centrality <http://www.sciencedirect.com/science/article/pii/S03788733
+          02000163>`_.
+        - `Wikipedia - Reachability <http://en.wikipedia.org/wiki/Reachability>`_
+        """
+
+        verts = ids
+
+        ## find the vertices within radius (and the path edges)
+        for i in range(radius):
+            edges_out = self.get_edges(src_ids=verts)
+            edges_in = self.get_edges(dst_ids=verts)
+
+            verts = list(edges_in['__src_id']) + list(edges_in['__dst_id']) + \
+                list(edges_out['__src_id']) + list(edges_out['__dst_id'])
+            verts = set(verts)
+
+
+        ## make a new graph to return and add the vertices
+        g = Graph()
+        g = g.add_vertices(self.get_vertices(verts), vid_field='__id')
+
+
+        ## add the requested edge set
+        if full_subgraph is True:
+            induced_edge_out = self.get_edges(src_ids=verts).to_dataframe()
+            induced_edge_in = self.get_edges(dst_ids=verts).to_dataframe()
+            df_induced = pd.concat((induced_edge_out, induced_edge_in))
+            df_induced.drop_duplicates(inplace=True)
+
+            flag_src = df_induced['__src_id'].isin(verts)
+            flag_dst = df_induced['__dst_id'].isin(verts)
+            flag = np.logical_and(flag_src, flag_dst)
+            edges = df_induced[flag]
+
+        else:
+            path_edges = pd.concat((edges_out.to_dataframe(),
+                                   edges_in.to_dataframe()))
+            path_edges.drop_duplicates(inplace=True)
+            edges = path_edges
+
+        g = g.add_edges(edges, src_field='__src_id', dst_field='__dst_id')
+        return g
+
 
-@shallow_throw
 def load_graph(filename, format='binary'):
     """
     Load graph from text file or previously saved graph binary.
@@ -858,15 +944,13 @@ def load_graph(filename, format='binary'):
     if format is 'binary' and not filename.endswith('.graph'):
         filename = filename + '.graph'
 
-    try:
+    with cython_context():
         if format is 'binary':
             proxy = glconnect.get_unity().load_graph(make_internal_url(filename))
         else:
             proxy = glconnect.get_unity().load_graph_from_url(make_internal_url(filename), format)
 
         return Graph(_proxy=proxy)
-    except IOError as e:
-        raise e
 
 
 def _vertex_list_to_dataframe(ls, id_column_name):
