@@ -5,13 +5,15 @@ ability to create, access and manipulate a remote scalable array object.
 SArray acts similarly to pandas.Series but without indexing.
 The data is immutable, homogeneous, and is stored on the GraphLab Server side.
 """
+import graphlab.connect as _mt
 import graphlab.connect.main as glconnect
-from graphlab.cython.sarray import UnitySArrayProxy
-from graphlab.util import make_internal_url, shallow_throw
+from graphlab.cython.cy_sarray import UnitySArrayProxy
+from graphlab.cython.context import debug_trace as cython_context
+from graphlab.util import make_internal_url
 import inspect
-import pandas, numpy
+import numpy
+import pandas
 import time
-import random
 
 __all__ = ['SArray']
 
@@ -19,13 +21,13 @@ class SArray(object):
     """
     SArray is an array object scaled to big data.
     The data in SArray is immutable, homogeneously typed, and is stored column wise.
-    SArray is also used as column in the :py:class:`~graphlab.SFrame`.
+    SArray is also used as column in the :py:class:`graphlab.SFrame`.
 
     SArray can be constructed in various ways:
 
     Construct an SArray from list.
 
-    >>> from graphlab import SArray 
+    >>> from graphlab import SArray
     >>> sa = SArray(data=[1,2,3,4,5], int)
 
     Construct an SArray from numpy.ndarray.
@@ -44,6 +46,17 @@ class SArray(object):
     >>> gl.SArray(data=pd.Series([1,2,3,4,5.0])).dtype()
     float
 
+    The Datatypes that are supported are, integer, float, string, and
+    list (a list of doubles). For instance:
+
+    Creates an SArray of strings:
+
+    >>> sa = SArray(data=['a','b'])
+
+    Creates an SArray of list:
+
+    >>> sa = SArray(data=[[1.0,1.0], [2.0,2.0]])
+
     Construct an SArray from local text file. (Only works for local server).
 
     >>> sf = SArray('/tmp/a_to_z.txt.gz')
@@ -56,6 +69,67 @@ class SArray(object):
 
     >>> sf = SArray('remote:///tmp/a_to_z.txt.gz')
 
+    SArrays support a large number of vectorized operations. For instance:
+
+    >>> sa = SArray([1,1,1,1,1])
+    >>> sb = SArray([2,2,2,2,2])
+    >>> sc = sa + sb
+    >>> sc
+    dtype: int
+    Rows: 5
+    [3, 3, 3, 3, 3]
+    >>> sc + 2
+    dtype: int
+    Rows: 5
+    [5, 5, 5, 5, 5]
+
+    Operators which are supported include all numeric operators (+,-,*,/), as
+    well as comparison operators (>, >=, <, <=), and logical operators (&, |).
+
+    For instance:
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> (sa >= 2) & (sa <= 4)
+    dtype: int
+    Rows: 5
+    [0, 1, 1, 1, 0]
+
+    We can also perform logical filtering operations:
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> sa[(sa >= 2) & (sa <= 4)]
+    dtype: int
+    Rows: 3
+    [2, 3, 4]
+
+    The numeric operators (+,-,*,/) also work on list types:
+
+    >>> sa = SArray(data=[[1.0,1.0], [2.0,2.0]])
+    >>> sa + 1
+    dtype: list
+    Rows: 2
+    [[2.0, 2.0], [3.0, 3.0]]
+    >>> sa + sa
+    dtype: list
+    Rows: 2
+    [[2.0, 2.0], [4.0, 4.0]]
+
+    The addition operator (+) can also be used for string concatenation:
+
+    >>> sa = SArray(data=['a','b'])
+    >>> sa + "x"
+    dtype: str
+    Rows: 2
+    ['ax', 'bx']
+
+    The SArray is also iterable, but not efficiently since this involves a
+    streaming transmission of data from the server to the client. This should
+    not be used for large data.
+
+    >>> sa = SArray([1,2,3,4,5])
+    >>> [i + 1 for i in sa]
+    [2, 3, 4, 5, 6]
+
     Parameters
     ----------
     data : list | numpy.ndarray | pandas.Series | string
@@ -94,12 +168,11 @@ class SArray(object):
     >>> sf = SArray("s3://mybucket/foo.csv") # works
     >>> graphlab.aws.teminate_EC2()
 
-    Similar restriction applies to :py:class:`~graphlab.Graph` and :py:class:`~graphlab.SFrame`.
+    Similar restriction applies to :py:class:`graphlab.Graph` and :py:class:`graphlab.SFrame`.
     """
 
-    @shallow_throw
     def __init__(self, data=[], dtype=None, ignore_cast_failure=False, _proxy=None):
-        """__init__(data=[], dtype=None, ignore_cast_failure=False)
+        """__init__(data=list(), dtype=None, ignore_cast_failure=False)
         Construct a new SArray. The source of data includes: list, numpy.ndarray, pandas.Series, and urls.
 
         Parameters
@@ -114,7 +187,7 @@ class SArray(object):
 
         dtype : SArray
             The data type of the SArray. Supported types are: {int,
-            float, str}.  If not specified (None), we attempt to evaluate
+            float, str, list}.  If not specified (None), we attempt to infer
             it from the input.  If it is a numpy array, or a Pandas
             series, the dtype of the array/series is used. If it is a
             list, it is first converted to a Pandas series, and the
@@ -133,6 +206,7 @@ class SArray(object):
         -----
         If data is pandas.Series, the index will be ignored.
         """
+        _mt._get_metric_tracker().track('sarray.init')
         if (_proxy):
             self.__proxy__ = _proxy
         else:
@@ -140,14 +214,18 @@ class SArray(object):
             # we need to perform type inference
             if dtype is None:
                 if (isinstance(data, list)):
-                    # if it is a list, convert the list to a pandas series
-                    # and get the type of that
-                    dtype = pandas.Series(data).dtype
+                    # if it is a list, Get the first type and make sure
+                    # the remaining items are all of the same type
+                    dtype = self.__infer_dtype_of_list__(data)
                 elif (isinstance(data, pandas.Series) or
                       isinstance(data, numpy.ndarray)):
                     # if it is a pandas series or numpy array, get the dtype
                     # of the series / array
                     dtype = data.dtype
+                    if dtype == object:
+                        # we need to get a bit more fine grained than that
+                        dtype = self.__infer_dtype_of_list__(data)
+
                 elif (isinstance(data, str)):
                     # if it is a file, we default to string
                     dtype = str
@@ -155,20 +233,51 @@ class SArray(object):
             if (isinstance(data, pandas.Series) or
                 isinstance(data, numpy.ndarray) or
                 isinstance(data, list)):
-                self.__proxy__.load_from_iterable(data, dtype, ignore_cast_failure)
+                with cython_context():
+                    self.__proxy__.load_from_iterable(data, dtype, ignore_cast_failure)
             elif (isinstance(data, str)):
                 if data.endswith('.sidx'):
                     internal_url = make_internal_url(data)
-                    self.__proxy__.load_from_sarray_index(internal_url)
+                    with cython_context():
+                        self.__proxy__.load_from_sarray_index(internal_url)
                 else:
                     internal_url = make_internal_url(data)
-                    self.__proxy__.load_from_url(internal_url, dtype)
+                    with cython_context():
+                        self.__proxy__.load_from_url(internal_url, dtype)
             else:
                 raise TypeError("Unexpected data source. " \
                                 "Possible data source types are: list, " \
                                 "numpy.ndarray, pandas.Series, and string(url)")
 
-    @shallow_throw
+    def __infer_dtype_of_list__(self, data):
+        # default is float
+        if len(data) == 0:
+            return float
+
+        unique_types = set([type(x) for x in data if x is not None])
+        if len(unique_types) == 0:
+            return float
+        # if dtype is list, recheck to make sure all values are numeric
+        elif len(unique_types) == 1:
+            dtype = unique_types.pop()
+            if dtype != list:
+                return dtype
+            else:
+                isvalid = [all([isinstance(elem, (int,long,float)) for elem in innerlist]) for innerlist in data]
+                if all(isvalid):
+                    return list
+                else:
+                    raise TypeError("Array type should have all entries as doubles")
+
+        elif len(unique_types) == 2:
+            # we can do both ints, longs, floats as a float
+            if unique_types.issubset(set([int,long,float])):
+                return float
+            else:
+                raise TypeError("Cannot infer Array type. Not all elements of array are the same type.")
+        else:
+            raise TypeError("Cannot infer Array type. Not all elements of array are the same type.")
+
     def save(self, targetfile):
         """
         Saves the SArray to file.
@@ -182,7 +291,8 @@ class SArray(object):
         if not targetfile.endswith('.sidx'):
             raise ValueError("Save target must end with sidx")
         else:
-            self.__proxy__.save(targetfile)
+            with cython_context():
+                self.__proxy__.save(targetfile)
 
     def __repr__(self):
         """
@@ -237,179 +347,189 @@ class SArray(object):
 
         return generator()
 
-    @shallow_throw
     def __add__(self, other):
         """
         If other is a scalar value, adds it to the current array, returning
         the new result. If other is an SArray, performs an element-wise
         addition of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '+'))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '+'))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '+'))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '+'))
 
-    @shallow_throw
     def __sub__(self, other):
         """
         If other is a scalar value, subtracts it from the current array, returning
         the new result. If other is an SArray, performs an element-wise
         subtraction of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '-'))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '-'))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '-'))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '-'))
 
-    @shallow_throw
     def __mul__(self, other):
         """
         If other is a scalar value, multiplies it to the current array, returning
         the new result. If other is an SArray, performs an element-wise
         multiplication of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '*'))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '*'))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '*'))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '*'))
 
-    @shallow_throw
     def __div__(self, other):
         """
         If other is a scalar value, divides each element of the current array
         by the value, returning the result. If other is an SArray, performs
         an element-wise division of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '/'))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '/'))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '/'))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '/'))
 
-    @shallow_throw
     def __lt__(self, other):
         """
         If other is a scalar value, compares each element of the current array
         by the value, returning the result. If other is an SArray, performs
         an element-wise comparison of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '<'))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '<'))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '<'))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '<'))
 
-    @shallow_throw
     def __gt__(self, other):
         """
         If other is a scalar value, compares each element of the current array
         by the value, returning the result. If other is an SArray, performs
         an element-wise comparison of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '>'))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '>'))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '>'))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '>'))
+
 
-    @shallow_throw
     def __le__(self, other):
         """
         If other is a scalar value, compares each element of the current array
         by the value, returning the result. If other is an SArray, performs
         an element-wise comparison of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '<='))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '<='))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '<='))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '<='))
+
 
-    @shallow_throw
     def __ge__(self, other):
         """
         If other is a scalar value, compares each element of the current array
         by the value, returning the result. If other is an SArray, performs
         an element-wise comparison of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '>='))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '>='))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '>='))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '>='))
+
 
-    @shallow_throw
     def __radd__(self, other):
         """
         Adds a scalar value to the current array.
         Returned array has the same type as the array on the right hand side
         """
-        return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '+'))
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '+'))
+
 
-    @shallow_throw
     def __rsub__(self, other):
         """
         Subtracts a scalar value from the current array.
         Returned array has the same type as the array on the right hand side
         """
-        return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '-'))
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '-'))
+
 
-    @shallow_throw
     def __rmul__(self, other):
         """
         Multiplies a scalar value to the current array.
         Returned array has the same type as the array on the right hand side
         """
-        return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '*'))
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '*'))
+
 
-    @shallow_throw
     def __rdiv__(self, other):
         """
         Divides a scalar value by each element in the array
         Returned array has the same type as the array on the right hand side
         """
-        return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '/'))
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.right_scalar_operator(other, '/'))
+
 
-    @shallow_throw
     def __eq__(self, other):
         """
         If other is a scalar value, compares each element of the current array
         by the value, returning the new result. If other is an SArray, performs
         an element-wise comparison of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '=='))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '=='))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '=='))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '=='))
+
 
-    @shallow_throw
     def __ne__(self, other):
         """
         If other is a scalar value, compares each element of the current array
         by the value, returning the new result. If other is an SArray, performs
         an element-wise comparison of the two arrays.
         """
-        if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '!='))
-        else:
-            return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '!='))
+        with cython_context():
+            if type(other) is SArray:
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '!='))
+            else:
+                return SArray(_proxy = self.__proxy__.left_scalar_operator(other, '!='))
+
 
-    @shallow_throw
     def __and__(self, other):
         """
         Perform a logical element-wise 'and' against another SArray.
         """
         if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '&'))
+            with cython_context():
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '&'))
         else:
             raise TypeError("SArray can only perform logical and against another SArray")
 
-    @shallow_throw
+
     def __or__(self, other):
         """
         Perform a logical element-wise 'or' against another SArray.
         """
         if type(other) is SArray:
-            return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '|'))
+            with cython_context():
+                return SArray(_proxy = self.__proxy__.vector_operator(other.__proxy__, '|'))
         else:
             raise TypeError("SArray can only perform logical or against another SArray")
 
-    @shallow_throw
+
     def __getitem__(self, other):
         """
         If other is an SArray of identical length, this function performs a
@@ -419,7 +539,8 @@ class SArray(object):
         if type(other) is SArray:
             if len(other) != len(self):
                 raise IndexError("Cannot perform logical indexing on arrays of different length.")
-            return SArray(_proxy = self.__proxy__.logical_filter(other.__proxy__))
+            with cython_context():
+                return SArray(_proxy = self.__proxy__.logical_filter(other.__proxy__))
         else:
             raise IndexError("Invalid type to use for indexing")
 
@@ -440,7 +561,7 @@ class SArray(object):
         """
         return self.__proxy__.dtype()
 
-    @shallow_throw
+
     def head(self, n=10):
         """
         Returns a pandas.DataFrame which contains the first n rows of the SFrame.
@@ -460,8 +581,65 @@ class SArray(object):
         """
         return self.__proxy__.head(n)
 
-    @shallow_throw
-    def apply(self, fn, dtype=None, skip_undefined=True, seed=time.time()):
+    def vector_slice(self, start, end=None):
+        """
+        If this SArray contains vectors, this returns a new SArray containing
+        each individual vector sliced, between start and end, exclusive.
+
+        For instance:
+
+        >>> g
+        dtype: list
+        Rows: 2
+        [[1.0, 2.0, 3.0],
+         [2.0, 3.0, 4.0]]
+
+        >>> g.vector_slice(0) # extracts the first element of each vector
+        dtype: float
+        Rows: 2
+        [1.0, 2.0]
+
+        >>> g.vector_slice(0, 2) # extracts the first two elements of each vector
+        dtype: list
+        Rows: 2
+        [[1.0, 2.0], [2.0, 3.0]]
+
+        If a vector cannot be sliced, the result will be None.
+
+        For instance:
+
+        >>> g
+        dtype: list
+        Rows: 3
+        [[1.0], [1.0, 2.0], [1.0, 2.0, 3.0]]
+
+        >>> g.vector_slice(2)
+        dtype: float
+        Rows: 3
+        [None, None, 3.0]
+
+        >>> g.vector_slice(0,2)
+        dtype: list
+        Rows: 3
+        [None, [1.0, 2.0], [1.0, 2.0]]
+
+        Parameters
+        ----------
+        start :  The start position of the slice.
+
+        end :  Optional. The end position of the slice. Note that the end
+        position is NOT included in the slice. Thus a g.vector_slice(1,3) will
+        extract entries in position 1 and 2.
+        """
+        if self.dtype() != list:
+            raise RuntimeError("Only Vector type can be sliced")
+        if end == None:
+            end = start + 1
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.vector_slice(start, end))
+
+    def apply(self, fn, dtype=None, skip_undefined=True, seed=None):
         """
         Returns a new SArray of dtype where each element in this SArray is
         transformed by fn(x).
@@ -493,19 +671,20 @@ class SArray(object):
             type ``dtype``
         """
         assert inspect.isfunction(fn), "Input must be a function"
-        try:
-            if dtype is None:
-                dtype = pandas.Series(self.head(100)).apply(fn).dtype
-        except Exception as e:
-            # convert all exceptions to RuntimeError.
-            # This provides uniformity of error types. (Since SArray
-            # basically just raises runtime errors)
-            raise RuntimeError(str(e))
+        dryrun = [fn(i) for i in self.head(100)]
+        if dtype == None:
+            dtype = self.__infer_dtype_of_list__(dryrun)
 
-        return SArray(_proxy=self.__proxy__.transform(fn, dtype, skip_undefined, seed))
+        if not seed:
+            seed = time.time()
 
-    @shallow_throw
-    def filter(self, fn, skip_undefined=True, seed=time.time()):
+        _mt._get_metric_tracker().track('sarray.apply')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.transform(fn, dtype, skip_undefined, seed))
+
+
+    def filter(self, fn, skip_undefined=True, seed=None):
         """
         Returns a new SArray which is filtered by the given function.
         If the lambda evaluates an element to true, this element is copied to the
@@ -528,10 +707,16 @@ class SArray(object):
         out : SArray
         """
         assert inspect.isfunction(fn), "Input must be a function"
-        return SArray(_proxy=self.__proxy__.filter(fn, skip_undefined, seed))
+        if not seed:
+            seed = time.time()
 
-    @shallow_throw
-    def sample(self, fraction, seed=time.time()):
+        _mt._get_metric_tracker().track('sarray.filter')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.filter(fn, skip_undefined, seed))
+
+
+    def sample(self, fraction, seed=None):
         """
         Returns an SArray which contains a subsample of the current SArray.
 
@@ -552,7 +737,13 @@ class SArray(object):
             raise ValueError('Invalid sampling rate: ' + str(fraction))
         if (self.size() == 0):
             return SArray()
-        return SArray(_proxy=self.__proxy__.sample(fraction, seed))
+        if not seed:
+            seed = time.time()
+
+        _mt._get_metric_tracker().track('sarray.sample')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.sample(fraction, seed))
 
     def _save_as_text(self, url):
         """
@@ -561,7 +752,7 @@ class SArray(object):
         raise NotImplementedError
 
 
-    @shallow_throw
+
     def all(self):
         """
         Returns True if all of the elements evaluate to True.
@@ -571,9 +762,10 @@ class SArray(object):
         -------
         out : bool
         """
-        return self.__proxy__.all()
+        with cython_context():
+            return self.__proxy__.all()
+
 
-    @shallow_throw
     def any(self):
         """
         Returns True if any of the elements evaluate to True.
@@ -583,12 +775,13 @@ class SArray(object):
         -------
         out : bool
         """
-        return self.__proxy__.any()
+        with cython_context():
+            return self.__proxy__.any()
+
 
-    @shallow_throw
     def max(self):
         """
-        Returns the maximum value in the SArray.  The empty frame returns numpy.nan.
+        Returns the maximum value in the SArray.  The empty frame returns None.
         Throws an exception if called on an SArray with non-numeric type.
 
         Returns
@@ -596,12 +789,13 @@ class SArray(object):
         out : type of SArray
             Maximum value of SArray
         """
-        return self.__proxy__.max()
+        with cython_context():
+            return self.__proxy__.max()
+
 
-    @shallow_throw
     def min(self):
         """
-        Returns the minimum value in the SArray.  The empty frame returns numpy.nan.
+        Returns the minimum value in the SArray.  The empty frame returns None.
         Throws an exception if called on an SArray with non-numeric type.
 
         Returns
@@ -609,27 +803,34 @@ class SArray(object):
         out : type of SArray
             Minimum value of SArray
         """
-        return self.__proxy__.min()
+        with cython_context():
+            return self.__proxy__.min()
+
 
-    @shallow_throw
     def sum(self):
         """
         Returns the sum of all the values in the sarray.  The empty frame
-        returns numpy.nan.  Throws an exception if called on an sarray with
-        non-numeric type.  Will overflow without warning.
+        returns None.
+        If the array contains strings, this will throws an exception.
+        If the array contains vectors, and all the vectors are the same length,
+        the sum over all the vectors will be returned. Otherwise, if the vectors
+        have different lengths, this will throw an exception.
+
+        For large values, this may overflow without warning.
 
         Returns
         -------
         out : type of SArray
             Sum of all values in SArray
         """
-        return self.__proxy__.sum()
+        with cython_context():
+            return self.__proxy__.sum()
+
 
-    @shallow_throw
     def mean(self):
         """
         Returns the mean of the values in the sarray.  The empty frame returns
-        numpy.nan.  Throws an exception if called on an sarray with non-numeric
+        None.  Throws an exception if called on an sarray with non-numeric
         type.
 
         Returns
@@ -637,13 +838,14 @@ class SArray(object):
         out : float
             Mean of all values in SArray
         """
-        return self.__proxy__.mean()
+        with cython_context():
+            return self.__proxy__.mean()
+
 
-    @shallow_throw
     def std(self, ddof=0):
         """
         Returns the standard deviation of all the values in the sarray as
-        a float.
+        a float.The empty array returns None.
 
         Parameters
         ----------
@@ -659,15 +861,15 @@ class SArray(object):
         -------
         out : float
             The standard deviation of all the values.
-            Returns numpy.nan if the sarray is empty.
         """
-        return self.__proxy__.std(ddof)
+        with cython_context():
+            return self.__proxy__.std(ddof)
+
 
-    @shallow_throw
     def var(self, ddof=0):
         """
         Returns the variance of all the values in the sarray as
-        a float.
+        a float. The empty array returns None.
 
         Parameters
         ----------
@@ -682,12 +884,24 @@ class SArray(object):
         Returns
         -------
         out : float
-            Variance of all values in SArray. Returns numpy.nan if the sarray
-            is empty.
+            Variance of all values in SArray.
         """
-        return self.__proxy__.var(ddof)
+        with cython_context():
+            return self.__proxy__.var(ddof)
+
+    def num_missing(self):
+        """
+        Returns the number of missing elements in the SArray.  Returns 0 if the
+        sarray is empty.
+
+        Returns
+        -------
+        out : int
+            Number of missing values.
+        """
+        with cython_context():
+            return self.__proxy__.num_missing()
 
-    @shallow_throw
     def nnz(self):
         """
         Returns the number of non-zero elements in the SArray.  Returns 0 if the
@@ -698,10 +912,10 @@ class SArray(object):
         out : int
             Number of non-zero elements.
         """
-        return self.__proxy__.nnz()
+        with cython_context():
+            return self.__proxy__.nnz()
 
-    @shallow_throw
-    def astype(self, dtype):
+    def astype(self, dtype, undefined_on_failure=False):
         """
         Returns a new SArray with all of the current values casted to the given
         type.
@@ -713,20 +927,30 @@ class SArray(object):
         dtype : type
             The type to cast the elements to in SArray
 
+        undefined_on_failure: bool
+            If set to true, runtime cast failures will be emitted as
+            missing values rather than failing.
+
         Returns
         -------
         out : SArray (of type dtype)
             The SArray converted to the dtype
         """
-        return SArray(_proxy=self.__proxy__.astype(dtype))
 
-    @shallow_throw
+        _mt._get_metric_tracker().track('sarray.astype', {'type':str(dtype)})
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.astype(dtype, undefined_on_failure))
+
     def clip(self, lower=numpy.nan, upper=numpy.nan):
         """
         Returns a new SArray with the clipped values of this SArray.
         This means to modify each value outside of the given bounds
         to be the bound.
 
+        This function can operate on numeric arrays, as well as vector arrays,
+        in which case each individual element in each vector is clipped.
+
         If lower or upper are given numpy.nan as the argument, this is interpreted
         as a non-existent bound.
 
@@ -747,12 +971,16 @@ class SArray(object):
         -------
         out : SArray
         """
-        return SArray(_proxy=self.__proxy__.clip(lower, upper))
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.clip(lower, upper))
 
     def clip_lower(self, threshold):
         """
         Returns a new SArray with all values clipped to the lower bound given.
 
+        This function can operate on numeric arrays, as well as vector arrays,
+        in which case each individual element in each vector is clipped.
+
         Parameters
         ----------
         threshold : float
@@ -767,13 +995,17 @@ class SArray(object):
         -------
         out : SArray
         """
-        return SArray(_proxy=self.__proxy__.clip(threshold, numpy.nan))
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.clip(threshold, numpy.nan))
+
 
-    @shallow_throw
     def clip_upper(self, threshold):
         """
         Returns a new SArray with all values clipped to the upper bound given.
 
+        This function can operate on numeric arrays, as well as vector arrays,
+        in which case each individual element in each vector is clipped.
+
         Parameters
         ----------
         threshold : float
@@ -788,24 +1020,9 @@ class SArray(object):
         -------
         out : SArray
         """
-        return SArray(_proxy=self.__proxy__.clip(numpy.nan, threshold))
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.clip(numpy.nan, threshold))
 
-#    def nonzero(self):
-#        """
-#        Returns a list of indices in this SArray that evaluate to a nonzero
-#
-#        Throws an exception if the SArray has at least one nan value. Care must
-#        be taken if the resulting list is big (larger than available memory).
-#
-#        Returns
-#        -------
-#        out : list
-#            The list of indices that correspond to each nonzero element in
-#            the SArray.
-#        """
-#        return self.__proxy__.nonzero()
-
-    @shallow_throw
     def tail(self, n=10):
         """
         Returns a list of the last n elements in the SArray.
@@ -824,9 +1041,10 @@ class SArray(object):
         out : list
             The last n elements of the SArray
         """
-        return self.__proxy__.tail(n)
+        with cython_context():
+            return self.__proxy__.tail(n)
+
 
-    @shallow_throw
     def dropna(self):
         """
         Returns a new SArray containing only the non-missing values of the
@@ -840,9 +1058,13 @@ class SArray(object):
         out : SArray
             The new SArray with missing values removed.
         """
-        return SArray(_proxy = self.__proxy__.drop_missing_values())
 
-    @shallow_throw
+        _mt._get_metric_tracker().track('sarray.dropna')
+
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.drop_missing_values())
+
+
     def topk_index(self, topk=10, reverse=False):
         """
         Used to return the topk elements, sorted by those elements (descending
@@ -867,5 +1089,22 @@ class SArray(object):
         -----
         This is mostly used internally by SFrame's topk function.
         """
-        return SArray(_proxy = self.__proxy__.topk_index(topk, reverse))
+        with cython_context():
+            return SArray(_proxy = self.__proxy__.topk_index(topk, reverse))
 
+    def sketch_summary(self):
+        """
+        Returns a graphlab.Sketch object which can be further queried for many
+        descriptive statistics over this SArray. Many of the statistics are
+        approximate. See the :class:`~graphlab.Sketch` documentation for more
+        detail.
+
+        Returns
+        -------
+        out : Sketch
+            Sketch object that contains descriptive statistics for this SArray.
+            Many of the statistics are approximate.
+        """
+        from graphlab.data_structures.sketch import Sketch
+        _mt._get_metric_tracker().track('sarray.sketch_summary')
+        return Sketch(self)
