@@ -7,16 +7,29 @@ and is stored column wise on the GraphLab Server side.
 """
 import graphlab.connect as _mt
 import graphlab.connect.main as glconnect
-from graphlab.cython.sframe import UnitySFrameProxy
-from graphlab.util import make_internal_url, shallow_throw
+from graphlab.cython.context import debug_trace as cython_context
+from graphlab.cython.cy_sframe import UnitySFrameProxy
+from graphlab.util import make_internal_url
 from graphlab.data_structures.sarray import SArray
+import graphlab.aggregate
+
+import inspect
 import pandas
 import time
 
+_has_matplotlib = False
+try:
+    import matplotlib.pyplot as plt
+    from mpl_toolkits.mplot3d import Axes3D
+    from matplotlib import rcParams
+    _has_matplotlib = True
+except:
+    pass
+
+
 __all__ = ['SFrame']
 
 
-@shallow_throw
 def load_sframe(filename):
     """
     Load an SFrame. The filename extension is used to determine the format
@@ -45,13 +58,13 @@ class SFrame(object):
 
     While the columns in the SFrame are immutable, SFrame itself is mutable in
     that columns can be added and subtracted from an SFrame with ease. Each
-    column of an SFrame is actually an :py:class:`~graphlab.SArray`, so an
+    column of an SFrame is actually an :py:class:`graphlab.SArray`, so an
     SFrame essentially acts as an ordered dict of SArrays.
 
     SFrame can be constructed in various ways. Currently, we support
     constructing an SFrame from:
 
-    * :py:class:`pandas.DataFrame`
+    * pandas.DataFrame
     * csv file (comma separated, first line is header)
     * sframe file (ends with .frame_idx)
     * general text file (with csv parsing options, See :py:meth:`read_csv()`)
@@ -69,11 +82,12 @@ class SFrame(object):
 
     >>> sf = SFrame(data='~/mydata/foo.csv')
 
-    Construct an SFrame from a csv file on Amazon S3.
-    This requires the environment variables: *AWS_ACCESS_KEY_ID* and *AWS_SECRET_ACCESS_KEY* to be set
-    before the python session started. Alternatively, you can use :py:func:`graphlab.aws.set_credentials()`
-    to set the credentials after python is started and :py:func:`graphlab.aws.get_credentials()` to
-    verify these environment variables.
+    Construct an SFrame from a csv file on Amazon S3. This requires the
+    environment variables: *AWS_ACCESS_KEY_ID* and *AWS_SECRET_ACCESS_KEY* to be
+    set before the python session started. Alternatively, you can use
+    :py:func:`graphlab.aws.set_credentials()` to set the credentials after
+    python is started and :py:func:`graphlab.aws.get_credentials()` to verify
+    these environment variables.
 
     >>> sf = SFrame(data='s3://mybucket/foo.csv')
 
@@ -81,7 +95,8 @@ class SFrame(object):
 
     >>> sf = SFrame(data='remote:///mydata/foo.csv.gz')
 
-    Construct an SFrame from a csv file downloaded from a URL using more csv parseing parameters.
+    Construct an SFrame from a csv file downloaded from a URL using more csv
+    parseing parameters.
 
     >>> sf = SFrame.read_csv('http://testdatasets.s3-website-us-west-2.amazonaws.com/users.csv.gz',
                              delimiter=',',
@@ -94,31 +109,55 @@ class SFrame(object):
     >>> sf.save('remote:///mysframe.frame_idx')
     >>> sf2 = graphlab.load_sframe('remote:///mysframe.frame_idx')
 
+    An SFrame is comprised of a collection of columns of SArrays, and individual
+    SArrays can be extracted easily. For instance given an SFrame:
+
+    >>> sf
+    Columns:
+        A	int
+        B	int
+    Rows: 3
+    Data:
+       A  B
+    0  1  4
+    1  2  5
+    2  3  6
+
+    The "A" column can be extracted using:
+
+    >>> sf["A"]
+    dtype: int
+    Rows: 3
+    [1, 2, 3]
+
+    And can be deleted using:
+
+    >>> del sf["A"]
+
     Parameters
     ----------
     data : Array | pandas.DataFrame | string
-        The actual interpretation
-        of this field is dependent on the "format" parameter. If data
-        is an Array or a Pandas DataFrame, the contents are stored in
-        the SFrame. If the contents is a string, it is interpreted as
-        a file. Files can be read from local file system, or
-        urls (file://, hdfs://, s3://, http://, or remote://)
+        The actual interpretation of this field is dependent on the "format"
+        parameter. If data is an Array or a Pandas DataFrame, the contents are
+        stored in the SFrame. If the contents is a string, it is interpreted as
+        a file. Files can be read from local file system, or urls (local://,
+        hdfs://, s3://, http://, or remote://)
 
     format : {'auto', 'array', 'dataframe', 'csv', 'sframe'}
-        The format of the data. Default 'auto' will automatically infer
-        the input data format. The inference rules are simple: If the
-        data is an array/dict or a dataframe, it is associated with
-        'array' and 'dataframe' respectively. If the data is a string,
-        it is interpreted as a file, and the file extension is used to
-        infer the file format.
+        The format of the data. Default 'auto' will automatically infer the
+        input data format. The inference rules are simple: If the data is an
+        array/dict or a dataframe, it is associated with 'array' and 'dataframe'
+        respectively. If the data is a string, it is interpreted as a file, and
+        the file extension is used to infer the file format.
 
 
     Notes
     -----
-    When working with the graphlab EC2 instance, e.g. :py:func:`graphlab.aws.launch_EC2()`,
-    SFrame cannot be constructed using local file path,  because it involves
-    potentially large amount of data transfer from client to server.
-    However, it is still ok to the remote file path.
+    When working with the graphlab EC2 instance, e.g.
+    :py:func:`graphlab.aws.launch_EC2()`, SFrame cannot be constructed using
+    local file path,  because it involves potentially large amount of data
+    transfer from client to server. However, it is still ok to the remote file
+    path.
 
     >>> graphlab.aws.launch_EC2('m1.large')
     >>> sf = SFrame('~/mydata/foo.csv') # throws exception
@@ -127,35 +166,34 @@ class SFrame(object):
     >>> sf = SFrame('s3://mybucket/foo.csv') # works
     >>> graphlab.aws.teminate_EC2()
 
-    Similar restriction applies to :py:class:`~graphlab.Graph` and :py:class:`~graphlab.SArray`.
+    Similar restriction applies to :py:class:`graphlab.Graph` and
+    :py:class:`graphlab.SArray`.
     """
 
     __slots__ = ['shape', '__proxy__']
 
-    @shallow_throw
+
     def __init__(self, data=[],
                  format='auto',
                  _proxy=None):
-        """__init__(data=[], format='auto')
+        """__init__(data=list(), format='auto')
         Construct a new SFrame from a url or a pandas.DataFrame.
 
         Parameters
         ----------
         data : Array | pandas.DataFrame | string
-            The actual interpretation
-            of this field is dependent on the "format" parameter. If data
-            is an Array or a Pandas DataFrame, the contents are stored in
-            the SFrame. If the contents is a string, it is interpreted as
-            a file. Files can be read from local file system, or
-            urls (file://, hdfs://, s3://, http://, or remote://)
+            The actual interpretation of this field is dependent on the "format"
+            parameter. If data is an Array or a Pandas DataFrame, the contents
+            are stored in the SFrame. If the contents is a string, it is
+            interpreted as a file. Files can be read from local file system, or
+            urls (local://, hdfs://, s3://, http://, or remote://)
 
         format : {'auto', 'array', 'dataframe', 'csv', 'sframe'}
-            The format of the data. Default 'auto' will automatically infer
-            the input data format. The inference rules are simple: If the
-            data is an array/dict or a dataframe, it is associated with
-            'array' and 'dataframe' respectively. If the data is a string,
-            it is interpreted as a file, and the file extension is used to
-            infer the file format.
+            The format of the data. Default 'auto' will automatically infer the
+            input data format. The inference rules are simple: If the data is an
+            array/dict or a dataframe, it is associated with 'array' and
+            'dataframe' respectively. If the data is a string, it is interpreted
+            as a file, and the file extension is used to infer the file format.
 
         _proxy : None
             Internal, do not use.
@@ -165,7 +203,7 @@ class SFrame(object):
         For CSV files, the preferred constructor is SFrame.read_csv since
         that has a lot more options which can be used to control the parser.
         """
-        # emit metrics for num_rows, num_columns, and type (file://, s3, hdfs, http)
+        # emit metrics for num_rows, num_columns, and type (local://, s3, hdfs, http)
         tracker = _mt._get_metric_tracker()
         if (_proxy):
             self.__proxy__ = _proxy
@@ -209,33 +247,33 @@ class SFrame(object):
 
             tracker.track(('sframe.format.%s' % _format), value=1)
 
-            if (_format == 'dataframe'):
-                self.__proxy__.load_from_dataframe(data)
-            elif (_format == 'array'):
-                pd = pandas.DataFrame(data)
-                pd.columns = [('X%d' % (i + 1)) for i in xrange(len(pd.columns))]
-                self.__proxy__.load_from_dataframe(pd)
-            elif (_format == 'csv' or _format == 'tsv'):
-                url = make_internal_url(data)
-                csv_config = dict()
-                if csv_delimiter is not None:
-                    csv_config['delimiter'] = csv_delimiter
-                    csv_config['use_header'] = True
-                self.__proxy__.load_from_csv(url, csv_config, dict())
-            elif (_format == 'sframe'):
-                if not data.endswith('.frame_idx'):
-                    data += '.frame_idx'
-                url = make_internal_url(data)
-                self.__proxy__.load_from_sframe_index(url)
-            else:
-                raise ValueError('Unknown input type: ' + format)
+            with cython_context():
+                if (_format == 'dataframe'):
+                    self.__proxy__.load_from_dataframe(data)
+                elif (_format == 'array'):
+                    pd = pandas.DataFrame(data)
+                    pd.columns = [('X%d' % (i + 1)) for i in xrange(len(pd.columns))]
+                    self.__proxy__.load_from_dataframe(pd)
+                elif (_format == 'csv' or _format == 'tsv'):
+                    url = make_internal_url(data)
+                    csv_config = dict()
+                    if csv_delimiter is not None:
+                        csv_config['delimiter'] = csv_delimiter
+                        csv_config['use_header'] = True
+                    self.__proxy__.load_from_csv(url, csv_config, dict())
+                elif (_format == 'sframe'):
+                    if not data.endswith('.frame_idx'):
+                        data += '.frame_idx'
+                    url = make_internal_url(data)
+                    self.__proxy__.load_from_sframe_index(url)
+                else:
+                    raise ValueError('Unknown input type: ' + format)
 
         self.shape = (self.num_rows(), self.num_cols())
         tracker.track('sframe.row.size', value=self.num_rows())
         tracker.track('sframe.col.size', value=self.num_cols())
 
     @classmethod
-    @shallow_throw
     def read_csv(cls,
                  url,
                  delimiter=',',
@@ -245,7 +283,7 @@ class SFrame(object):
                  double_quote=True,
                  quote_char='\"',
                  skip_initial_space=True,
-                 column_type_hints={},
+                 column_type_hints=str,
                  verbose=True):
         """
         Constructs an SFrame from a CSV file.
@@ -274,12 +312,20 @@ class SFrame(object):
         skip_initial_space : bool
             If extra spaces at the start of a field is ignored
 
-        column_type_hints : dict[string, type]
-            This provides type hints for
-            each column. For instance {'user':int} will hint that the column
-            called "user" should be parsed as an integer. If no hint is given
-            for a certain column, that column's type is assumed to be a string.
+        column_type_hints : type, list[type], dict[string, type]
+            This provides type hints for each column.
             Supported types are int, float, str
+            - If a single type is provided, the type will be
+            applied to all columns. For instance, column_type_hints=float will
+            force all columns to be parsed as float.
+            - If a list of types is provided, the types applies
+            to each column in order, e.g.[int, float, str]
+            will parse the first column as int, second as float and third as string.
+            - If a dictionary of column name to type is provided,
+            each type value in the dictonary is applied to the key it belongs to.
+            For instance {'user':int} will hint that the column
+            called "user" should be parsed as an integer, and the rest will default
+            to string.
 
         verbose : bool
             If True, print the progress.
@@ -298,7 +344,27 @@ class SFrame(object):
 
         if (not verbose):
             glconnect.get_client().set_log_progress(False)
-        proxy.load_from_csv(internal_url, parsing_config, column_type_hints)
+
+        if type(column_type_hints) is type:
+            type_hints = {'__all_columns__': column_type_hints}
+        elif type(column_type_hints) is list:
+            type_hints = dict(zip(['__X%d__' % i for i in range(len(column_type_hints))], column_type_hints))
+        elif type(column_type_hints) is dict:
+            type_hints = column_type_hints
+
+
+        _mt._get_metric_tracker().track('sframe.csv.parse')
+
+        suffix=''
+        if url.find('://') == -1:
+            suffix = 'local'
+        else:
+            suffix = url.split('://')[0]
+
+        _mt._get_metric_tracker().track(('sframe.location.%s' % (suffix)), value=1)
+
+        with cython_context():
+            proxy.load_from_csv(internal_url, parsing_config, type_hints)
         glconnect.get_client().set_log_progress(True)
 
         return cls(_proxy=proxy)
@@ -361,7 +427,8 @@ class SFrame(object):
         if type(other) is SArray:
             if len(other) != len(self):
                 raise IndexError("Cannot perform logical indexing on arrays of different length.")
-            return SFrame(_proxy=self.__proxy__.logical_filter(other.__proxy__))
+            with cython_context():
+                return SFrame(_proxy=self.__proxy__.logical_filter(other.__proxy__))
 
     def dtype(self):
         """
@@ -474,8 +541,62 @@ class SFrame(object):
         """
         return self.__proxy__.tail(n)
 
-    @shallow_throw
-    def sample(self, fraction, seed='default'):
+    def apply(self, fn, dtype=None, seed=None):
+        """
+        Returns a new SArray of dtype where each element in this SArray is
+        transformed by fn(x) where x is a row in the sframe, as a dictionary.
+        The fn should return a value which can be cast into dtype.
+
+        If dtype is not specified, the first 100 rows of the SFrame are
+        used to make a guess of the target datatype.
+
+        Parameters
+        ----------
+        fn : function
+            The function to transform each row of the sframe. The return
+            type should be convertible to dtype if dtype is not None.
+
+        dtype : dtype
+            The dtype of the new SArray. If None, the first 100
+            elements of the array are used to guess the target
+            data type.
+
+        seed : int, optional
+            Used as the seed if a random number generator is included in fn.
+
+        Returns
+        -------
+        out : SArray
+            The SArray transformed by fn.  Each element of the SArray is of
+            type ``dtype``
+
+
+        Example
+        -------
+
+        >>> import graphlab
+        >>> sf = graphlab.SFrame.read_csv('netflix.csv')
+
+        The following code create a new SArray where each element is the
+        string concatination of the 'user_id', 'movie_id' and 'rating' columns.
+
+        >>> sa = sf.apply(lambda x: str(x['user_id']) + str(x['movie_id']) + str(x['rating']))
+
+        """
+        assert inspect.isfunction(fn), "Input must be a function"
+        dryrun = [fn(dict(zip(self.column_names(), i))) for i in self.head(10).values]
+        if dtype is None:
+            dtype = SArray(dryrun).dtype()
+
+        if not seed:
+            seed = int(time.time())
+
+        _mt._get_metric_tracker().track('sframe.apply')
+
+        with cython_context():
+            return SArray(_proxy=self.__proxy__.transform(fn, dtype, seed))
+
+    def sample(self, fraction, seed=None):
         """
         Returns an SFrame which contains a subsample (row) of the current SFrame.
 
@@ -493,18 +614,21 @@ class SFrame(object):
         out : SFrame
             An SFrame containing the subsampled rows of the SFrame.
         """
-        if (seed is 'default'):
-            seed = time.time()
+        if not seed:
+            seed = int(time.time())
 
         if (fraction > 1 or fraction < 0):
             raise ValueError('Invalid sampling rate: ' + str(fraction))
+
+        _mt._get_metric_tracker().track('sframe.sample')
+
         if (self.num_rows() == 0 or self.num_cols() == 0):
             return self
         else:
-            return SFrame(_proxy=self.__proxy__.sample(fraction, seed))
+            with cython_context():
+                return SFrame(_proxy=self.__proxy__.sample(fraction, seed))
 
-    @shallow_throw
-    def random_split(self, fraction, seed=time.time()):
+    def random_split(self, fraction, seed=None):
         """
         Returns a pair of SFrames from random splitting the current one. The first
         sframe contains uniformly random "fraction" rows, and the second
@@ -527,10 +651,22 @@ class SFrame(object):
         if (self.num_rows() == 0 or self.num_cols() == 0):
             return (SFrame(), SFrame())
 
-        proxy_pair = self.__proxy__.random_split(fraction, seed)
-        return (SFrame(data=[], _proxy=proxy_pair[0]), SFrame(data=[], _proxy=proxy_pair[1]))
+        if not seed:
+            seed = int(time.time())
+
+        # The server side requires this to be an int, so cast if we can
+        try:
+            seed = int(seed)
+        except ValueError:
+            raise ValueError('The \'seed\' parameter must be of type int.')
+
+        _mt._get_metric_tracker().track('sframe.random_split')
+
+        with cython_context():
+            proxy_pair = self.__proxy__.random_split(fraction, seed)
+            return (SFrame(data=[], _proxy=proxy_pair[0]), SFrame(data=[], _proxy=proxy_pair[1]))
+
 
-    @shallow_throw
     def topk(self, column_name, k=10, reverse=False):
         """
         Returns the topk rows sorted by the column_name in descending order.
@@ -553,12 +689,15 @@ class SFrame(object):
         """
         if type(column_name) is not str:
             raise TypeError("column_name must be a string")
+
+        _mt._get_metric_tracker().track('sframe.topk')
+
         df = self[self[column_name].topk_index(k, reverse)].to_dataframe()
         df = df.sort(column_name, ascending=reverse)
-        df.index = range(k)
+        df.index = range(len(df))
         return df
 
-    @shallow_throw
+
     def save(self, filename, format='binary'):
         """
         Save the SFrame to file.
@@ -573,6 +712,9 @@ class SFrame(object):
         format : {'binary', 'csv'}, optional
             Format in which to save the SFrame.
         """
+
+        _mt._get_metric_tracker().track('sframe.save', properties={'format':format})
+
         if filename.endswith(('.csv', '.csv.gz')):
             format = 'csv'
         elif filename.endswith('.frame_idx'):
@@ -588,15 +730,16 @@ class SFrame(object):
         ## Save the SFrame
         url = make_internal_url(filename)
 
-        if format is 'binary':
-            assert filename.endswith('.frame_idx')
-            self.__proxy__.save(url)
+        with cython_context():
+            if format is 'binary':
+                assert filename.endswith('.frame_idx')
+                self.__proxy__.save(url)
 
-        elif format is 'csv':
-            assert filename.endswith(('.csv', '.csv.gz'))
-            self.__proxy__.save_as_csv(url)
-        else:
-            raise ValueError("Unsupported format: {}".format(format))
+            elif format is 'csv':
+                assert filename.endswith(('.csv', '.csv.gz'))
+                self.__proxy__.save_as_csv(url)
+            else:
+                raise ValueError("Unsupported format: {}".format(format))
 
     def select_column(self, key):
         """
@@ -617,7 +760,8 @@ class SFrame(object):
         """
         if not isinstance(key, str):
             raise TypeError("Invalid key type: must be str")
-        return SArray(data=[], _proxy=self.__proxy__.select_column(key))
+        with cython_context():
+            return SArray(data=[], _proxy=self.__proxy__.select_column(key))
 
     def select_columns(self, keylist):
         """
@@ -644,7 +788,8 @@ class SFrame(object):
             raise TypeError("keylist must be an iterable")
         if not all([isinstance(x, str) for x in keylist]):
             raise TypeError("Invalid key type: must be str")
-        return SFrame(data=[], _proxy=self.__proxy__.select_columns(keylist))
+        with cython_context():
+            return SFrame(data=[], _proxy=self.__proxy__.select_columns(keylist))
 
     def add_column(self, data, name=""):
         """
@@ -664,7 +809,9 @@ class SFrame(object):
             raise TypeError("Must give column as SArray")
         if not isinstance(name, str):
             raise TypeError("Invalid column name: must be str")
-        self.__proxy__.add_column(data.__proxy__, name)
+        with cython_context():
+            self.__proxy__.add_column(data.__proxy__, name)
+            self.shape = (self.num_rows(), self.num_cols())
 
     def add_columns(self, datalist, namelist):
         """
@@ -687,7 +834,9 @@ class SFrame(object):
             raise TypeError("Must give column as SArray")
         if not all([isinstance(x, str) for x in namelist]):
             raise TypeError("Invalid column name in list: must all be str")
-        self.__proxy__.add_columns([x.__proxy__ for x in datalist], namelist)
+        with cython_context():
+            self.__proxy__.add_columns([x.__proxy__ for x in datalist], namelist)
+            self.shape = (self.num_rows(), self.num_cols())
 
     def remove_column(self, name):
         """
@@ -699,7 +848,9 @@ class SFrame(object):
             The name of the column to remove.
         """
         colid = self.column_names().index(name)
-        self.__proxy__.remove_column(colid)
+        with cython_context():
+            self.__proxy__.remove_column(colid)
+            self.shape = (self.num_rows(), self.num_cols())
 
     def swap_columns(self, column_1, column_2):
         """
@@ -716,7 +867,8 @@ class SFrame(object):
         colnames = self.column_names()
         colid_1 = colnames.index(column_1)
         colid_2 = colnames.index(column_2)
-        self.__proxy__.swap_columns(colid_1, colid_2)
+        with cython_context():
+            self.__proxy__.swap_columns(colid_1, colid_2)
 
     def rename(self, names):
         """
@@ -731,9 +883,10 @@ class SFrame(object):
         """
         if (type(names) is not dict):
             raise TypeError('names must be a dictionary: oldname -> newname')
-        for k in names:
-            colid = self.column_names().index(k)
-            self.__proxy__.set_column_name(colid, names[k])
+        with cython_context():
+            for k in names:
+                colid = self.column_names().index(k)
+                self.__proxy__.set_column_name(colid, names[k])
 
     def __getitem__(self, key):
         """
@@ -767,14 +920,18 @@ class SFrame(object):
                 elif hasattr(value, '__iter__'):
                     self.add_column(SArray(value), key)
                 else:
-                    self.__proxy__.add_column_from_const(key, value)
+                    with cython_context():
+                        self.__proxy__.add_column_from_const(key, value)
             else:
                 # add the column to a unique column name.
                 tmpname = '__' + '-'.join(self.column_names())
                 if (type(value) is SArray):
                     self.add_column(value, tmpname)
+                elif hasattr(value, '__iter__'):
+                    self.add_column(SArray(value), tmpname)
                 else:
-                    self.__proxy__.add_column_from_const(tmpname, value)
+                    with cython_context():
+                        self.__proxy__.add_column_from_const(tmpname, value)
                 # if add succeeded, remove the column name and rename tmpname->columnname.
                 self.swap_columns(key, tmpname)
                 self.remove_column(key)
@@ -792,6 +949,9 @@ class SFrame(object):
         """
         Provides an iterator to the rows of the sframe.
         """
+
+        _mt._get_metric_tracker().track('sframe.__iter__')
+
         def generator():
             elems_at_a_time = 262144
             self.__proxy__.begin_iterator()
@@ -807,3 +967,154 @@ class SFrame(object):
                     break
 
         return generator()
+
+    def group(self, column):
+        """
+        Return a new SFrame where the rows are grouped according to the
+        value of the provided column.
+
+        Parameters
+        ----------
+        column : string
+            Name of column to group.
+
+        Returns
+        -------
+        out_sf : SFrame
+            A new SFrame having the same shape and data, but the rows
+            are grouped by the given column.
+
+        Notes
+        -----
+        The group method does not sort on the selected column.
+        """
+        _mt._get_metric_tracker().track('sframe.group')
+        if column not in self.column_names():
+            raise RuntimeError("Column " + column + " does not exist in SFrame")
+        else:
+            with cython_context():
+                return SFrame(_proxy=self.__proxy__.group(column))
+
+    def groupby(self, key_columns, operation_dict, *args):
+        """
+        Perform a group on the key_columns followed by aggregations on the
+        columns listed in operation_dict.
+
+        The operation_dict parameter is a dictionary that indicates which
+        aggregation operators to use and which columns to use them on. The
+        available operators are SUM, MAX, MIN, COUNT, AVG, VAR, STDV, and
+        QUANTILE. For convenience, aggregators MEAN, STD, and VARIANCE are
+        available as synonyms for AVG, STDV, and VAR. See
+        :mod:`~graphlab.aggregate` for more detail on the aggregators.
+
+        The columns of the output SFrame are named "[operator] of
+        [aggregation column name]". For example, if key_columns is 'user_id' and
+        operation_dict is {'rating': [gl.aggregate.SUM, gl.aggregate.COUNT]},
+        the output is an SFrame with three columns: 'user_id', 'Sum of rating',
+        and 'Count of rating'.
+
+        The COUNT aggregate is special and can be used without specifying the
+        full operation_dict parameter.
+
+        Parameters
+        ----------
+        key_columns : string | list[string]
+            Column(s) to group by.
+
+        operation_dict : dict
+            Dictionary of columns and aggregation operations. Each key is a
+            column name and each value is a list of aggregators to be applied to
+            that particular column.
+
+        Returns
+        -------
+        out_sf : SFrame
+            A new SFrame, with a column for each groupby column and each
+            aggregation operation.
+
+        Examples
+        --------
+        >>> import graphlab as gl
+        >>> import graphlab.aggregate as agg
+        >>> sf = gl.load_sframe('netflix.frame_idx')
+
+        Compute the number of occurrences of each user.
+
+        >>> user_count = sf.groupby(key_columns='user_id',
+                                    operation_dict={'user_id': agg.COUNT})
+
+        Compute the same thing with the special syntax for the COUNT aggregate.
+
+        >>> user_count = sf.groupby(key_columns='user_id',
+                                    operation_dict=agg.COUNT)
+
+        Compute the mean and standard deviation of ratings per user.
+
+        >>> user_rating_stats = sf.groupby(key_columns='user_id',
+                                           operation_dict={'rating': agg.MEAN, agg.STD]})
+
+        Compute the count, mean, and standard deviation of ratings per (user,
+        time).
+
+        >>> user_rating_stats = sf.groupby(['user_id', 'time'], {'rating':
+                                           [agg.COUNT, agg.AVG, agg.STDV]})
+
+        The groupby function can take a variable length list of aggregation
+        specifiers so if we want the count and the 0.25 and 0.75 quantiles of
+        ratings:
+
+        >>> user_rating_stats = sf.groupby(['user_id', 'time'], agg.COUNT,
+                                           {'rating': agg.QUANTILE(0.25, 0.75)})
+        """
+        # some basic checking first
+        # make sure key_columns is a list
+        if isinstance(key_columns, str):
+            key_columns = [key_columns]
+        # check that every column is a string, and is a valid column name
+        my_column_names = self.column_names()
+        key_columns_array = []
+        for column in key_columns:
+            if not isinstance(column, str):
+                raise TypeError("Column name must be a string")
+            if column not in my_column_names:
+                raise KeyError("Column " + column + " does not exist in SFrame")
+            key_columns_array.append(column)
+
+        group_columns = []
+        group_ops = []
+
+        all_ops = [operation_dict] + list(args)
+
+        for operation in all_ops:
+            # if instance type is not dict, then it must be gl.aggregate.COUNT
+            if not isinstance(operation, dict):
+                if isinstance(operation, str) and operation == graphlab.aggregate.COUNT:
+                    #translate a single COUNT value to graphlab.aggregate.COUNT
+                    operation = {'':graphlab.aggregate.COUNT}
+                else:
+                    raise RuntimeError("Groupby operations must be either a dictionary of key:operation, or graphlab.aggregate.COUNT")
+
+
+            # now sweep the dict and add to group_columns and group_ops
+            for key in operation:
+                val = operation[key]
+                if type(key) is not list:
+                    key = [key]
+                if type(val) is not list:
+                    val = [val]
+                all_pairs = [(k, v) for k in key for v in val]
+                group_columns = group_columns + [k for (k, v) in all_pairs]
+                group_ops = group_ops + [v for (k, v) in all_pairs]
+
+        # let's validate group_columns and group_ops are valid
+        for (col, op) in zip(group_columns, group_ops):
+            if not isinstance(col, str):
+                raise TypeError("Column name must be a string")
+            elif not isinstance(op, str):
+                raise TypeError("Operation type not recognized.")
+            else:
+                _mt._get_metric_tracker().track('sframe.groupby', properties={'operator':op})
+                if op is not graphlab.aggregate.COUNT and col not in my_column_names:
+                    raise KeyError("Column " + column + " does not exist in SFrame")
+        with cython_context():
+            return SFrame(_proxy=self.__proxy__.groupby_aggregate(key_columns_array, group_columns, group_ops))
