@@ -7,7 +7,19 @@ information.
 import graphlab.toolkits.main as main
 from graphlab.toolkits.model import Model
 from graphlab.data_structures.sframe import SFrame
-from pandas import DataFrame
+from graphlab.data_structures.sarray import SArray
+from pandas import DataFrame, unique
+import math
+
+_has_matplotlib = False
+try:
+    # import matplotlib as mpl
+    import matplotlib.pyplot as plt
+    from mpl_toolkits.mplot3d import Axes3D
+    from matplotlib import rcParams
+    _has_matplotlib = True
+except:
+    pass
 
 
 class KmeansModel(Model):
