@@ -97,7 +97,7 @@ def create(graph, reset_prob=0.15, threshold=0.01, verbose=False, plot=False):
         print "Starting pagerank computation."
         verbose = 0
 
-    opts = {'threshold': threshold, 'reset_prob': reset_prob, 'graph': graph.__proxy__}
+    opts = {'threshold': threshold, 'random_jump_prob': reset_prob, 'graph': graph.__proxy__}
     params = main.run('pagerank', opts, verbose, plot)
     return PagerankModel({'threshold': threshold, 'reset_prob': reset_prob},
                          params['model'])
