@@ -1,8 +1,9 @@
 """
 """
-
+import graphlab.connect as _mt
 import graphlab as _graphlab
 from graphlab.toolkits.model import Model
+from graphlab.data_structures.sarray import SArray as _SArray
 from graphlab.data_structures.sframe import SFrame as _SFrame
 import pandas as _pd
 import json as _json
@@ -48,6 +49,8 @@ class RecommenderModel(Model):
 
     def summary(self):
         """Returns a summary of training statistics."""
+        _mt._get_metric_tracker().track('toolkit.recsys.summary')
+
         return self.training_stats()
 
     def __str__(self):
@@ -88,7 +91,7 @@ class RecommenderModel(Model):
             ret += "\nSchema\n"
             for k in ["user_column", "item_column", "target_column"]:
                 if k in options.keys():
-                    if options[k] not in ['', 'not_specified']:
+                    if options[k] not in ['', 'not_specified'] and options[k] is not None:
                         ret += "  %-14s %s\n" % \
                             (k.replace('_', ' ') + ':', options[k])
                     del options[k]
@@ -217,6 +220,7 @@ class RecommenderModel(Model):
             A DataFrame with model predictions, with columns user_id, item_id, and
             score
         """
+        _mt._get_metric_tracker().track('toolkit.recsys.predict')
 
         if type(dataset) == _pd.DataFrame:
             dataset = _SFrame(dataset)
@@ -229,7 +233,7 @@ class RecommenderModel(Model):
         result = _SFrame(None, _proxy=response['data'])
         return result
 
-    def recommend(self, dataset, k=30):
+    def recommend(self, dataset, users=None, k=30):
         """
         Return the k best recommend items for each user, excluding items for
         which the user already has an entry. Typically this is used after the
@@ -239,14 +243,6 @@ class RecommenderModel(Model):
         >>> m.train(data, 'user', 'item')
         >>> recs = m.recommend(data)
 
-        The returned object has four columns: user, item, score, and rank (0 is
-        the highest). The observations in **dataset** are automatically excluded
-        from the returned recommendations. In some situations we may have new
-        observations that we want to be able to use to make recommendations, and
-        the recommend function accepts a *new* dataset to acoommodate this.
-
-        >>> recs = m.recommend(new_data)
-
         Parameters
         ----------
         dataset : SFrame/pandas.DataFrame
@@ -255,6 +251,9 @@ class RecommenderModel(Model):
             dataset, and all observations in this data set will be excluded from
             the recommendations.
 
+        users : SArray
+            If provided, recommendations will only be returned for this set of users.
+
         k : int, optional
             The number of items to return for each user (default is 30).
 
@@ -264,14 +263,22 @@ class RecommenderModel(Model):
             A SFrame with the top ranked items for each user. The
             columns are: *user*, *item*, *score*, and *rank*.
         """
+        _mt._get_metric_tracker().track('toolkit.recsys.recommend')
 
         if type(dataset) == _pd.DataFrame:
             dataset = _SFrame(dataset)
         elif type(dataset) != _SFrame:
             raise NotImplemented
 
+        # TEMP: Get unique list of users on Python side
+        if users is None:
+            users = list(set(dataset[self.get('user_column')]))
+        users_sf = _SFrame()
+        users_sf['users'] = users
+
         opt = {}
         opt["model"] = self.__proxy__
+        opt["users"] = users_sf
         opt["top_k"] = k
         opt['available_data'] = dataset
         response = _graphlab.toolkits.main.run('recsys_recommend', opt)
@@ -288,6 +295,8 @@ class RecommenderModel(Model):
         out : dict
             Statistics about model training, e.g. runtime.
         """
+        _mt._get_metric_tracker().track('toolkit.recsys.training_stats')
+
         opts = {'model': self.__proxy__}
         response = _graphlab.toolkits.main.run("recsys_get_stats", opts)
         return response
@@ -310,6 +319,8 @@ class RecommenderModel(Model):
             data used for internal validation.
         """
 
+        _mt._get_metric_tracker().track('toolkit.recsys.get_validation_split')
+
         if type(dataset) == _pd.DataFrame:
             dataset = _SFrame(dataset)
         elif type(dataset) != _SFrame:
@@ -353,7 +364,9 @@ class RecommenderModel(Model):
         elif type(dataset) != _SFrame:
             raise NotImplementedError
         if skip_set is None:
-            skip_set = _SFrame(_pd.DataFrame())
+            skip_set = _SFrame()
+            for colname in dataset.column_names():
+                skip_set[colname] = _SArray()
         if type(skip_set) == _pd.DataFrame:
             skip_set = _SFrame(skip_set)
         elif type(skip_set) != _SFrame:
@@ -445,8 +458,10 @@ class RecommenderModel(Model):
             these metrics for each user at several cutoff values.
         """
 
+        _mt._get_metric_tracker().track('toolkit.recsys.evaluate')
+
         # If the model does not have a target column, compute prec-recall.
-        if self.get('target_column') == '':
+        if self.get('target_column') is None or self.get('target_column') == '':
             results = self._evaluate_precision_recall(dataset, **kwargs)['results']
             if verbose:
                 print "\nPrecision and recall summary statistics by cutoff"
